#' Scorecards calculate means of metrics
#' 
#'@description Scorecards function to calculate the mean of the loaded metrics 
#'
#'@example
#SC_metric_means <- SC_load_metrics(loaded_metrics,
#                                       system.filename = list('ecmwfs5','dwds21'),
#                                       reference.filename = 'era5',
#                                       start.year = 1993,
#                                       end.year = 2016,
#                                       metrics = c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr') 
#                                       region)



SC_metric_means <- function(loaded_metrics,
                                     system.filename, 
                                     reference.filename, 
                                     start.year, 
                                     end.year,
                                     metrics,
                                     region){
    
    loaded_metrics <- loaded_metrics
    ncores = 4
  
    ## Start months
    start.months <- c("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12") ## TO DO: use month.abb function
    n.start.months <- length(start.months)
    
    ## Forecast months 
    ftime.months <- 1:6 #indices(recipe$Analysis$Time$ftime_min : recipe$Analysis$Time$ftime_max)


    ## Define regions
    # region.names <- initialfunction_regionnames_and_borders()[,1]
    # region.names.figure <- initialfunction_regionnames_and_borders()[,2]
    lat.min <- -90 #initialfunction_regionnames_and_borders()[,3]
    lat.max <- 90 #initialfunction_regionnames_and_borders()[,4]
    lon.min <- 0 #c() #initialfunction_regionnames_and_borders()[,5]
    lon.max <- 360 #c() #initialfunction_regionnames_and_borders()[,6]
    # n.regions <- length(region.names)
  

    mean.metrics <- list() 
    
    file_dim <- NULL
    
    ## TO DO: Loop over region
    
    all_metric_means <- list()
    for (sys in 1:length(system.filename)) {
      means_by_ref <- list()
      for (ref in 1:length(reference.filename)) {
        means_by_ftime <- list()
        for (ft in 1:length(ftime.months)){ ## DOUBT: Need to loop over ftime ?? 
          means_by_sdate <- list()
          for(m in 1:length(start.months)){
            met_means <- list ()
             for(met in 1:length(metrics)){
  
                weighted.mean <- multiApply::Apply(data = loaded_metrics[[sys]][[ref]][, , ft, met, m], 
                                                   target_dims = c('lon','lat'),
                                                   fun = ClimProjDiags::WeightedMean,
                                                   lon = as.vector(attributes(loaded_metrics[[sys]][[ref]])$lon), 
                                                   lat = as.vector(attributes(loaded_metrics[[sys]][[ref]])$lat),
                                                   mask = NULL, londim = 'lon', latdim = 'lat',
                                                   region = c(lon_min,lon_max,lat_min,lat_max),
                                                   ncores = ncores)$output1
            
                
                ## Save data for each metric
                if (is.null(file_dim)) {
                  file_dim <- names(dim(weighted.mean))
                  met_means <- abind::abind(met_means, weighted.mean, along = 5) ## ?
                } ## close if
              } ## close loop on metric
                
              means_by_sdate <- abind::abind(means_by_sdate, met_means, along = 4) ## ?
            } ## close loop on start months
        
            means_by_ftime <- abind::abind(means_by_ftime, means_by_sdate, along = 3) ## ?
          } ## close loop on ftime
        
          means_by_ref <- abind::abind(means_by_ref, means_by_ftime, along = 2) ## ?
        } ## close loop on reference
        
        all_metric_means <- abind::abind(all_metric_means, means_by_ref, along = 1) ## ?
      } ## close loop on system
    
    ### TO DO: Round the scores ??
      
    return(all_metric_means)
      
} ## close function
    


    #   ## position of lat and lon min and max values inside the netCDF
    #   pos.lat.min <- pos.lat.max <- pos.lon.min <- pos.lon.max <- c()
    #   for(i in 1:n.regions){
    #     if(is.null(lat.min)) pos.lat.min[i] <- which(lat.netCDF == min(lat.netCDF)) else pos.lat.min[i] <- initialfunction_position_latlon(as.integer(lat.min[i]), lat.netCDF)
    #     if(is.null(lat.max)) pos.lat.max[i] <- which(lat.netCDF == max(lat.netCDF)) else pos.lat.max[i] <- initialfunction_position_latlon(as.integer(lat.max[i]), lat.netCDF)
    #     if(is.null(lon.min)) pos.lon.min[i] <- which(lon.netCDF == min(lon.netCDF)) else pos.lon.min[i] <- initialfunction_position_latlon(as.integer(lon.min[i]), lon.netCDF)
    #     if(is.null(lon.max)) pos.lon.max[i] <- which(lon.netCDF == max(lon.netCDF)) else pos.lon.max[i] <- initialfunction_position_latlon(as.integer(lon.max[i]), lon.netCDF)
    #   }
    # 
    #   ## Empty arrays with the score values to plot:
    #   mean.bias <- mean.corr <- mean.crpss <- mean.rpss <- mean.enssprerr <- array(NA, c(n.start.months, n.regions, n.ftime.months))
    # 
    #   for(i in 1:n.regions){
    #     for(m in start.months){
    #       mm <- as.numeric(m)
    #       
    #       ## Select only scores inside the target region: [lon, lat, n.ftime.months]
    #       bias.region <- bias[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
    #       corr.region <- corr[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
    #       crpss.region <- crpss[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
    #       rpss.region <- rpss[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
    #       enssprerr.region <- enssprerr[[mm]][pos.lon.min[i]:pos.lon.max[i], pos.lat.max[i]:pos.lat.min[i], my.ftime.months]
    #       
    #       ## Replace values with (-)Inf with NA or the average will be (-)Inf too: (but infinite values should be absent in theory)
    #       rpss.region[which(is.infinite(rpss.region))] <- NA 
    #       crpss.region[which(is.infinite(crpss.region))] <- NA 
    #       enssprerr.region[which(is.infinite(enssprerr.region))] <- NA 
    #       
    #       ## Average score values inside the target region: 
    #       mean.bias[mm, i,] <- apply(bias.region, 3, mean, na.rm = TRUE) ## [n.start.months, n.regions, n.ftime.months]
    #       mean.corr[mm, i,] <- apply(corr.region, 3, mean, na.rm = TRUE)
    #       mean.rpss[mm, i,] <- apply(rpss.region, 3, mean, na.rm = TRUE)
    #       mean.crpss[mm, i,] <- apply(crpss.region, 3, mean, na.rm = TRUE)
    #       mean.enssprerr[mm, i,] <- apply(enssprerr.region, 3, mean, na.rm = TRUE)
    #       
    #     } ## close if on start.months
    #   } ## close if on n.regions
    # #} ## close if on scorecard_type    
    # 
    # 
    # ## Round the scores
    # mean.bias <- round(mean.bias, n.dec)
    # mean.corr <- round(mean.corr, 2)
    # mean.crpss <- round(mean.crpss, 2)
    # mean.rpss <- round(mean.rpss, 2)
    # mean.enssprerr <- round(mean.enssprerr, 2)
    

    
    
      